// nameserverDoc.cpp : implementation of the CNameserverDoc class
//

#include "stdafx.h"
#include "nameserver.h"

#include "cltUtenti.h"
#include "lstUtenti.h"
#include "cltStanze.h"
#include "lstStanze.h"
#include "nameMsg.h"
#include "nameserverView.h"
#include "nameserverDoc.h"
#include "CheckConn.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNameserverDoc

IMPLEMENT_DYNCREATE(CNameserverDoc, CDocument)

BEGIN_MESSAGE_MAP(CNameserverDoc, CDocument)
	//{{AFX_MSG_MAP(CNameserverDoc)
	ON_COMMAND(ID_VIEW_STANZE, OnViewStanze)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNameserverDoc construction/destruction

CNameserverDoc::CNameserverDoc()
{
	// TODO: add one-time construction code here
	m_pListenStanze = NULL;
	m_pListenUtenti = NULL;
	m_pCheckConn = NULL;
	
}

CNameserverDoc::~CNameserverDoc()
{
	delete m_pListenStanze;
	delete m_pListenUtenti;
}

// Inizializza le socket che ascoltano (listening) sulla porta degli utenti e su quella delle stanze
// inizializza inoltre la socket che effettua gli aggiornamenti dei record
BOOL CNameserverDoc::OnNewDocument()
{
	CString strTemp;

	if (!CDocument::OnNewDocument())
		return FALSE;

	((CEditView*)m_viewList.GetHead())->SetWindowText(NULL);
	((CNameserverView*)m_viewList.GetHead())->SetCheckTimer(CHECK_TIME);
	
	CTime time = CTime::GetCurrentTime();
	SetTitle(time.Format("Avviato %A, %B %d, %Y"));
	

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	if(strTemp.LoadString(IDS_SERVER_INIT_OK))
				Visualizza(strTemp);
			
	// inizializzazione porta stanze, TCP
	m_pListenStanze = new CListeningStanze(this);
	if (m_pListenStanze->Create(PORTA_STANZE))
	{
		if (!m_pListenStanze->Listen())
		{
			if(strTemp.LoadString(IDS_SERVER_INIT_FAIL))
				Visualizza(strTemp);
			return FALSE;
		}
	}

	// inizializzazione porta Utenti, TCP
	m_pListenUtenti = new CListeningUtenti(this);
	if (m_pListenUtenti->Create(PORTA_UTENTI))
	{
		if (!m_pListenUtenti->Listen())
		{
			if(strTemp.LoadString(IDS_SERVER_INIT_FAIL))
				Visualizza(strTemp);
			return FALSE;
		}
	}

	// inizializzazione porta di controllo delle stanze, UDP, uso lo stesso numero di porta, tanto  UDP
	m_pCheckConn = new CCheckConn(this);
	if (!m_pCheckConn->Create(PORTA_STANZE,SOCK_DGRAM) || !m_pCheckConn->Init())
	{
		if(strTemp.LoadString(IDS_SERVER_INIT_FAIL))
				Visualizza(strTemp);
		return FALSE;
	}

	
	
	if(strTemp.LoadString(IDS_SERVER_INIT_COMPLETED))
		Visualizza(strTemp);
		
	return true;
}



/////////////////////////////////////////////////////////////////////////////
// CNameserverDoc serialization

void CNameserverDoc::Serialize(CArchive& ar)
{
	// CEditView contains an edit control which handles all serialization
	((CEditView*)m_viewList.GetHead())->SerializeRaw(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CNameserverDoc diagnostics

#ifdef _DEBUG
void CNameserverDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CNameserverDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNameserverDoc commands

void CNameserverDoc::AccettaStanza()
{
	CClientStanze* pSocket = new CClientStanze(this);

	if (m_pListenStanze->Accept(*pSocket)) pSocket->Init();
	else delete pSocket;
}

void CNameserverDoc::AccettaUtenti()
{
	CClientUtenti* pSocket = new CClientUtenti(this);

	if (m_pListenUtenti->Accept(*pSocket)) pSocket->Init();
	else delete pSocket;
}

void CNameserverDoc::RiceviStanza(CClientStanze *pSocket)
{
	CNameMsg* pMsg = new CNameMsg;
	CString strTemp;
	int nAns = IDM_CONFIRM;

	try
	{
		// operazioni idempotenti
		do {
			pSocket->ReceiveMsg(pMsg);

			if (pMsg->m_nPort == IDM_LOGOUT)
			{
				if(DeleteStanza(pMsg))
					if(strTemp.LoadString(IDS_LOGOUT_OK))
						Visualizza(strTemp + ": " + pMsg->m_strName + "\r\n");
				nAns = IDM_LOGOUT;
				delete pMsg;

			}
			else if (RisolviStanza(pMsg,pSocket), TrovaStanza(pMsg,TRUE) == NULL)
			{
				
				if(strTemp.LoadString(IDS_LOGON_OK))
					Visualizza(strTemp);
				
				ImpostaT2L(pMsg);

				VisualizzaMsg(*pMsg);

				m_StanzeList.AddTail(pMsg);			
			}
			else
			{
				if(strTemp.LoadString(IDS_LOGON_FAIL))
					Visualizza(strTemp);
				nAns = IDM_ERROR;

				delete pMsg;
			}
		} while (!pSocket->m_pIn->IsBufferEmpty());

	}
	catch(CFileException e)
	{
		CString strTemp;
		if (strTemp.LoadString(IDS_READ_ERROR))
			Visualizza(strTemp);
		nAns = IDM_ERROR;
	}
	
	try
	{	
		CNameMsg ansMsg(strTemp,NULL,nAns);
		pSocket->SendMsg(&ansMsg);
	}
	catch(CFileException e)
	{
		CString strTemp;
		if (strTemp.LoadString(IDS_WRITE_ERROR))
			Visualizza(strTemp);
	}

	
}

void CNameserverDoc::RiceviUtente(CClientUtenti *pSocket)
{
	CNameMsg* pMsg = new CNameMsg;
	CNameMsg* pNameMsg;
	CString strTemp;

	do {
		try
		{
			
				pSocket->ReceiveMsg(pMsg);

				pNameMsg = TrovaStanza(pMsg,FALSE);

				if (pNameMsg)
				{
					if(strTemp.LoadString(IDS_RICERCA_OK))
						Visualizza(strTemp);

					VisualizzaMsg(*pNameMsg);
					
				}
				else
				{
					if(strTemp.LoadString(IDS_RICERCA_FAIL))
						Visualizza(strTemp);
					pNameMsg = pMsg;
					pMsg->m_strName=strTemp;
					pMsg->m_nPort=IDM_ERROR;
				}

			
		}
		catch(...)
		{
			CString strTemp;
			if (strTemp.LoadString(IDS_READ_ERROR))
				Visualizza(strTemp);
			pNameMsg = pMsg;
			pMsg->m_strName=strTemp;
			pMsg->m_nPort=IDM_ERROR;
		}
	
		try
		{	
			pSocket->SendMsg(pNameMsg);
		}
		catch(...)
		{
			CString strTemp;
			if (strTemp.LoadString(IDS_WRITE_ERROR))
				Visualizza(strTemp);
		}
	} while (!pSocket->m_pIn->IsBufferEmpty());

	delete pSocket;
	delete pMsg;
}

void CNameserverDoc::DeleteContents() 
{
	// TODO: Add your specialized code here and/or call the base class
	delete m_pListenStanze;
	m_pListenStanze = NULL;

	delete m_pListenUtenti;
	m_pListenUtenti = NULL;

	delete m_pCheckConn;
	m_pCheckConn = NULL;

	m_StanzeList.RemoveAll();

	if (!m_viewList.IsEmpty())
		((CEditView*)m_viewList.GetHead())->SetWindowText(_T(""));

	CDocument::DeleteContents();
}

inline void CNameserverDoc::Visualizza(LPCTSTR testo)
{
	((CNameserverView*)m_viewList.GetHead())->AppendiTesto(testo);
}

CNameMsg *CNameserverDoc::TrovaStanza(CNameMsg *const pMsg, BOOL checkAdd) const
{
	POSITION pos;
	for(pos = m_StanzeList.GetHeadPosition(); pos != NULL;)
	{
		CNameMsg* pNameMsg = (CNameMsg*)m_StanzeList.GetNext(pos);
		
		BOOL flag = pMsg->m_strAdd==pNameMsg->m_strAdd && pMsg->m_nPort==pNameMsg->m_nPort;
		// il concetto di uguaglianza dipende dall'override dell'operatore == per la classe CNameMsg
		
		if (*pMsg == *pNameMsg || (checkAdd && flag)) 
			return pNameMsg;
	}
	return NULL;
}

BOOL CNameserverDoc::DeleteStanza(CNameMsg *pMsg)
{
	POSITION pos,temp;
	for(pos = m_StanzeList.GetHeadPosition(); pos != NULL;)
	{
		temp=pos;
		CNameMsg* pNameMsg = (CNameMsg*)m_StanzeList.GetNext(pos);
		// il concetto di uguaglianza dipende dall'override dell'operatore == per la classe CNameMsg
		if (*pMsg == *pNameMsg) 
		{
			m_StanzeList.RemoveAt(temp);
			return TRUE;
		}

	}
	return FALSE;
}


void CNameserverDoc::VisualizzaStanze()
{
	POSITION pos;
	Visualizza("Stanze presenti sul name server:");
	for(pos = m_StanzeList.GetHeadPosition(); pos != NULL;)
	{
		CNameMsg* pNameMsg = (CNameMsg*)m_StanzeList.GetNext(pos);
		Visualizza(pNameMsg->m_strName);
	}
	Visualizza("");
}

void CNameserverDoc::OnViewStanze() 
{
	// TODO: Add your command handler code here
	VisualizzaStanze();
}

void CNameserverDoc::VisualizzaMsg(CNameMsg &msg)
{
	CString numStanza;
	numStanza.Format("%d",msg.m_nPort+PORTA_BASE);
	Visualizza((CString) "Estremi stanza: " + msg.m_strName + " -> " + msg.m_strAdd + ":" + numStanza);
	Visualizza((CString) "Descrizione: " + msg.m_strDesc);
	Visualizza(msg.m_T2L.Format("Record in vita fino a %A, %B %d, %Y, %H:%M:%S\r\n")); 
}


// la funzione Risolvi stanza completa il CNameMsg relativo ad una data connessione aggiungedogli anche i dati sul pari, se questi non sono presenti
inline void CNameserverDoc::RisolviStanza(CNameMsg *pMsg, CClientStanze *pSocket)
{
	if (pMsg->m_strAdd==_T(""))
	{
		CString addr;
		UINT port;

		pSocket->GetPeerName(addr, port);
		
		pMsg->m_strAdd=addr;
	}

}

void CNameserverDoc::CheckT2L()
{
	CTime t = CTime::GetCurrentTime();
	// Visualizza("Controllo time to live:");

	POSITION pos, temp;
	for(pos = m_StanzeList.GetHeadPosition(); pos != NULL;)
	{
		temp = pos;
		CNameMsg* pNameMsg = (CNameMsg*)m_StanzeList.GetNext(pos);
		if (pNameMsg->m_T2L<t) //  scaduto
		{
			Visualizza(pNameMsg->m_strName + "  scaduta, controllo in corso...");
			if (pNameMsg->m_nTries>0) // non risponde
			{
				pNameMsg->m_nTries--;
				VERIFY(CheckStanza(pNameMsg)!=SOCKET_ERROR);
				CString temp;
				temp.Format("(%d tentativi rimasti)\r\n", pNameMsg->m_nTries);
				Visualizza(temp);
			}
			else 
			{
				m_StanzeList.RemoveAt(temp); 
				Visualizza("Tutti i tentativi falliti: Rimozione stanza\r\n");
			}
		}
	}	
}


int CNameserverDoc::CheckStanza(CNameMsg *pMsg)
{
	UINT port = pMsg->m_nPort+PORTA_BASE;
	UINT len = pMsg->m_strName.GetLength()+1;

	return m_pCheckConn->SendTo((LPCTSTR) pMsg->m_strName, len, port, pMsg->m_strAdd);
}


BOOL CNameserverDoc::AggiornaT2L(CNameMsg &msg)
{
	CNameMsg* pMsg = TrovaStanza(&msg,TRUE);
	
	if (pMsg)
	{
		ImpostaT2L(pMsg); 
		return TRUE;
	}
	else return FALSE; // non esiste
}

// negoziazione time to live e numero di tentativi (tries)
void CNameserverDoc::ImpostaT2L(CNameMsg *pMsg)
{
		
#ifdef _DEBUG
		CTimeSpan ts(0,0,1,0);
		pMsg->m_T2L = CTime::GetCurrentTime() + ts;
		pMsg->m_nTries = NUM_TRIES;		
#else
		// se il tempo di vita  troppo ridotto viene impostato al MIN_TIME2LIVE	
		CTimeSpan ts(0,0,MIN_TIME2LIVE,0);
		if (pMsg->m_TS2L<ts) pMsg->m_TS2L = ts;
		pMsg->m_T2L = CTime::GetCurrentTime() + pMsg->m_TS2L;
		pMsg->m_nTries = NUM_TRIES;
#endif

}


